/********************************************************
 Name          : oled.c
 Author        : MoroboshiLUM
 Copyright     :
 Description   : EVK1101 template
 **********************************************************/

// Include Files
#include <avr32/io.h>
#include "compiler.h"
#include "board.h"

/* TODO: Add software framework include drivers below */
#include "gpio.h"
#include "spi.h"
#include "rtc.h"
#include "pm.h"

// wait is timeout * 62us
static void wait(int timeout)
{
	volatile unsigned int start,now;

	start=rtc_get_value(&AVR32_RTC);
	while (1) {
	  now=rtc_get_value(&AVR32_RTC);
	  now=(now-start) & 0x7FFFFFFF;
	  if (now>timeout) break;
	}
}

// send command to OLED
static void send_cmd(unsigned char cmd)
{
	gpio_clr_gpio_pin(AVR32_PIN_PB04);		// cmd select
	while (spi_write(&AVR32_SPI,cmd)!=SPI_OK) ;
}

// send data to OLED
static void send_data(unsigned char data)
{
	gpio_set_gpio_pin(AVR32_PIN_PB04);		// data select
	while (spi_write(&AVR32_SPI,data)!=SPI_OK) ;
}

static void init_oled(void)
{
	wait(2);
    gpio_set_gpio_pin(AVR32_PIN_PA10);		// RESET de-Assert
    gpio_clr_gpio_pin(AVR32_PIN_PB05);		// VDD ON
	wait(1000); // 600ms wait
	gpio_clr_gpio_pin(AVR32_PIN_PB04);		// command select
	spi_selectChip(&AVR32_SPI,2); 			// select cs2

	send_cmd(0xAE); // Display Off
	send_cmd(0x15); // Column Address
	send_cmd(0x00); // Set
	send_cmd(0x5F); // Set
	send_cmd(0x75); // Row Addess
	send_cmd(0x00); // Set
	send_cmd(0x3F); // Set
	send_cmd(0x81); // Set Contrast for Color A
	send_cmd(0x5A); // Set
	send_cmd(0x82); // Set Contrast for Color B
	send_cmd(0x2C); // Set
	send_cmd(0x83); // Set Contrast for Color C Now setting value is 15V, 100cd.
	send_cmd(0x54); // Set If you want to change
	send_cmd(0x87); // Master Current Control the OLED Brightnees.
	send_cmd(0x09); // Set Pls see the Note 1)
	send_cmd(0xA0); // Set Re-map & Data Format
//	send_cmd(0x70); // Set
	send_cmd(0x72); // Set
	send_cmd(0xA1); // Set Display Start Line
	send_cmd(0x00); // Set
	send_cmd(0xA2); // Set off Set
	send_cmd(0x00); // Set
	send_cmd(0xA8); // Multiplex Ratio
	send_cmd(0x3F); // Set
	send_cmd(0xAD); // Master Configuration
	send_cmd(0x8E); // Set
	send_cmd(0xB1); // Adjust Driving period
	send_cmd(0x11); // Set
	send_cmd(0xB3); // Colock Divider Oscillator Frequency
	send_cmd(0xD0); // Set
	send_cmd(0xBB); // Pre_charge Voltage Setting
	send_cmd(0x00); // Set
	send_cmd(0xBC); // Pre_charge Voltage Setting Now setting value is 15V, 100cd.
	send_cmd(0x2A); // Set If you want to change
	send_cmd(0xBD); // Pre_charge Voltage Setting the OLED Brightnees.
	send_cmd(0x00); // Set Pls see the Note 1)
	send_cmd(0xBE); // Set VCOMH Level
	send_cmd(0x3F); // Set
	send_cmd(0xB9); // Enable Linear Gray Scale
	send_cmd(0xAF); // Display On
	send_cmd(0x15); //Set Column Address RAM
	send_cmd(0x00); //Dec->0
	send_cmd(0x5F); //Dec->95
	send_cmd(0x75); //Set Row RAM Address
	send_cmd(0x00); //Dec->0
	send_cmd(0x3F); //Dec->63
}

static void init_gpio(void)
{
  gpio_set_gpio_pin(AVR32_PIN_PB05);		// VDD OFF
  gpio_clr_gpio_pin(AVR32_PIN_PA10);		// RESET Assert
  gpio_clr_gpio_pin(AVR32_PIN_PB04);		// command select

  gpio_enable_pin_glitch_filter(GPIO_PUSH_BUTTON_0);
}

static void init_rtc(void)
{
  rtc_init(&AVR32_RTC,RTC_OSC_32KHZ, 0); // treat as 16KHz Counter
   // Enable the RTC
  rtc_enable(&AVR32_RTC);
}

void init_oled_disp(void) {
	/* TODO: replace this comment with your code */
	init_gpio();
	init_rtc();
	init_oled();

	spi_unselectChip(&AVR32_SPI,2);			// unselect chip
	gpio_set_gpio_pin(AVR32_PIN_PB04);		// data select
}

void disp_off(void)
{
  spi_unselectChip(&AVR32_SPI,2);
  gpio_clr_gpio_pin(AVR32_PIN_PB04);		// command select
  spi_selectChip(&AVR32_SPI,2);

  send_cmd(0xAE); // Display Off
  gpio_set_gpio_pin(AVR32_PIN_PB05);		// VDD OFF
  wait(1000); // 600ms wait
}

// buffer = B,G,R data,
void disp_data(unsigned char *buf)
{
	int i;
	unsigned short r,g,b,outdata;
	unsigned char lodata,hidata;

	send_cmd(0x15); //Set Column Address RAM
	send_cmd(0x00); //Dec->0
	send_cmd(0x5F); //Dec->95
	send_cmd(0x75); //Set Row RAM Address
	send_cmd(0x00); //Dec->0
	send_cmd(0x3F); //Dec->63
	spi_unselectChip(&AVR32_SPI,2);			// unselect chip
	gpio_set_gpio_pin(AVR32_PIN_PB04);		// data select
	spi_selectChip(&AVR32_SPI,2);			// select chip

	for (i=0;i<96*64;i++) {
	  b=((*buf++) & 0xf8) << 8;
	  g=((*buf++) & 0xfc) << 3;
	  r=((*buf++) >> 3) & 0x1f;
	  outdata=b | g | r;
	  hidata=outdata >> 8;
	  lodata=outdata & 0xff;
	  while (spi_write(&AVR32_SPI,hidata)!=SPI_OK) ;
	  while (spi_write(&AVR32_SPI,lodata)!=SPI_OK) ;
	}

	spi_unselectChip(&AVR32_SPI,2);			// unselect chip
}
